// 67d7842dbbe25473c3c32b93c0da8047785f30d78e8a024de1b57352245f9689
/*******************************************************************************
 * Copyright (c) 2007, 2012 Wind River Systems, Inc. and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License 2.0
 * which accompanies this distribution, and is available at
 * https://www.eclipse.org/legal/epl-2.0/
 *
 * Contributors:
 *     Wind River Systems - initial API and implementation
 *******************************************************************************/

/*******************************************************************************
 * Modified by Xilinx,Inc
 * Copyright 2022 Xilinx, Inc. All Rights Reserved. Modifications
 * are made available under the terms of the Eclipse Public License 2.0
 * which accompanies this distribution, and is available at
 * https://www.eclipse.org/legal/epl-2.0/
 *
 *******************************************************************************/

package org.eclipse.tcf.internal.dap.command;

import org.eclipse.tcf.commands.IDebugCommandRequest;
import org.eclipse.tcf.commands.IResumeHandler;
import org.eclipse.tcf.dap.IStatus;
import org.eclipse.tcf.internal.dap.TCFContextState;
import org.eclipse.tcf.internal.dap.TCFModel;
import org.eclipse.tcf.internal.dap.TCFNode;
import org.eclipse.tcf.internal.dap.TCFNodeExecContext;
import org.eclipse.tcf.internal.dap.actions.TCFAction;
import org.eclipse.tcf.protocol.IChannel;
import org.eclipse.tcf.protocol.IErrorReport;
import org.eclipse.tcf.protocol.IToken;
import org.eclipse.tcf.services.IRunControl;
import org.eclipse.tcf.util.TCFDataCache;

import java.util.logging.Logger;


public class ResumeCommand extends StepCommand implements IResumeHandler {

    protected static final Logger LOGGER = Logger.getLogger(ResumeCommand.class.getSimpleName());

    public ResumeCommand(TCFModel model) {
        super(model);
    }

    @Override
    protected boolean canExecute(IRunControl.RunControlContext ctx) {
        return ctx.canResume(IRunControl.RM_RESUME);
    }

    @Override
    protected void execute(final IDebugCommandRequest monitor,
            final IRunControl.RunControlContext ctx, boolean src_step, final Runnable done) {
        new TCFAction(model.getLaunch(), ctx.getID()) {
            boolean cmd_sent;
            public void run() {
                if (cmd_sent) {
                    TCFNode node = model.getNode(ctx_id);
                    if (node instanceof TCFNodeExecContext) {
                        TCFDataCache<TCFContextState> state = ((TCFNodeExecContext)node).getState();
                        if (!state.validate(this)) return;
                    }
                    done();
                    return;
                }
                cmd_sent = true;
                ctx.resume(IRunControl.RM_RESUME, 1, new IRunControl.DoneCommand() {
                    public void doneCommand(IToken token, Exception error) {
                        if (error != null && model.getChannel().getState() == IChannel.STATE_OPEN) {
                            if (error instanceof IErrorReport) {
                                IErrorReport r = (IErrorReport)error;
                                if (r.getErrorCode() == IErrorReport.TCF_ERROR_ALREADY_RUNNING) {
                                    run();
                                    return;
                                }
                            }
                            launch.removeContextActions(getContextID());
                            LOGGER.severe("Cannot resume: " + error.getLocalizedMessage());
                            //TODO: Fix this
//                            monitor.setStatus(new Status(IStatus.ERROR,
//                                    Activator.PLUGIN_ID, IStatus.OK, "Cannot resume: " + error.getLocalizedMessage(), error));
                        }
                        run();
                    }
                });
            }
            public void done() {
                super.done();
                done.run();
            }
        };
    }
}
